<?php

/**
 * @author Targetbay Team
 * @copyright Copyright (c) 2016 Targetbay
 * @package Targetbay_Tracking
 */

namespace Targetbay\Tracking\Model;

use Targetbay\Tracking\Api\ListProductInterface;

/**
 * Defines the implementation class of the ListProductInterface.
 */
class ListProducts implements ListProductInterface
{
    // Product type configurable.
    const CONFIGURABLE_PRODUCT = 'configurable';
    const BUNDLE_PRODUCT = 'bundle';
    const GROUPED_PRODUCT = 'grouped';

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $productFactory;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Targetbay\Tracking\Helper\Data
     */
    protected $trackingHelper;

    /**
     * @var \Magento\CatalogInventory\Api\StockStateInterface
     */
    protected $stockState;
    protected $request;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory
     */
    protected $productCollection;

    /**
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Targetbay\Tracking\Helper\Data $trackingHelper
     * @param \Magento\CatalogInventory\Api\StockStateInterface $stockState
     * @param \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollection
     */
    public function __construct(
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Targetbay\Tracking\Helper\Data $trackingHelper,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\CatalogInventory\Api\StockStateInterface $stockState,
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollection
    ) {
        $this->_productFactory = $productFactory;
        $this->_storeManager = $storeManager;
        $this->request = $request;
        $this->_trackingHelper = $trackingHelper;
        $this->_stockState = $stockState;
        $this->_productCollection = $productCollection;
    }

    /**
     * Get the Products with pagination
     *
     * @return products
     */
    public function getList(\Magento\Framework\Api\SearchCriteriaInterface $searchCriteria)
    {

        $storeId = $this->request->getParam('store_id');

        $collection = $this->_productCollection->create()->addAttributeToSelect('*');

        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());
        $collection->addStoreFilter($storeId);

        $products = $collection->load()->toArray();

        foreach ($products as $id => $data) {
            $product = $this->_productFactory->create()->load($id);
            $categoryIds = $product->getCategoryIds();
            
            if (!empty($product->getImage()) && $product->getImage() !== 'no_selection') {
                $imageUrl = $this->_storeManager->getStore()->getBaseUrl(
                    \Magento\Framework\UrlInterface::URL_TYPE_MEDIA
                );
                $imageUrl .='catalog/product';
                $imageUrl .= $product->getImage();
            } else {
                $imageUrl = '';
            }

            // Get product url key
            if (!empty($product->getUrlKey())) {
                $urlKey = $product->getUrlKey();
            } else {
                $urlKey = $product->getProductUrl();
            }

            $products[$id]['image_url'] = $imageUrl;
            $products[$id]['category_id'] = implode(',', $categoryIds);
            $products[$id]['stock_count'] = $this->_stockState->getStockQty(
                $product->getId(),
                $product->getStore()->getWebsiteId()
            );
            $products[$id]['abstract'] = $product->getAbstract();
            $products[$id]['currency'] = $this->_storeManager->getStore()->getCurrentCurrencyCode();
            $products[$id]['visibility'] = $product->getVisibility();
            $products[$id]['status'] = $product->getStatus();
            $products[$id]['url_key'] = $urlKey;
            $products[$id]['website_id'] = $product->getWebsiteIds();
            $products[$id]['store_id'] = $product->getStoreIds();
            $products[$id]['price'] = $product->getFinalPrice();
            $products[$id]['special_price'] = $product->getSpecialPrice();

            if (count($product->getRelatedProductIds()) > 0) {
                $products[$id]['related_product_id'] = implode(',', $product->getRelatedProductIds());
            }
            if (count($product->getUpSellProductIds()) > 0) {
                $products[$id]['upsell_product_id'] = implode(',', $product->getUpSellProductIds());
            }
            if (count($product->getCrossSellProductIds()) > 0) {
                $products[$id]['crosssell_product_id'] = implode(',', $product->getCrossSellProductIds());
            }

            $configOptions = [];
            $customOptions = [];
            $childProductData = [];

            switch ($product->getTypeId()) {
                case self::CONFIGURABLE_PRODUCT:
                    if ($productAttributeOptions = $this->_productFactory->create()->load($product->getId())
                        ->getTypeInstance(true)->getConfigurableAttributesAsArray($product)) {
                        $configOptions = $this->_trackingHelper->productOptions($productAttributeOptions, 'label');
                    }

                    $childProducts = $product->getTypeInstance()->getUsedProductIds($product);
                    foreach ($childProducts as $childProductId) {
                        $childProductDetails = $this->_productFactory->create()->load($childProductId);
                        if ($childProductDetails->getVisibility() == 1) {
                            $childProductData[$childProductId] = $this->_trackingHelper->getProductData($childProductDetails);
                            $childProductData[$childProductId]['parent_id'] = $product->getId();
                        }
                    }
                    $products[$id]['child_items'] = $childProductData;
                    $products[$id]['parent_id'] = $product->getId();
                    break;
                case self::BUNDLE_PRODUCT:
                    $collection = $product->getTypeInstance(true)
                        ->getSelectionsCollection(
                            $product->getTypeInstance(true)->getOptionsIds($product),
                                                    $product
                        );

                    foreach ($collection as $item) {
                        $childProductId = $item->getId();
                        $childProductDetails = $this->_productFactory->create()->load($item->getId());
                        if ($childProductDetails->getVisibility() == 1) {
                            $childProductData[$childProductId] = $this->_trackingHelper->getProductData(
                                                                                    $childProductDetails
                        );
                            $childProductData[$childProductId]['parent_id'] = $product->getId();
                        }
                    }
                    $products[$id]['child_items'] = $childProductData;
                    $products[$id]['parent_id'] = $product->getId();
                    break;
                case self::GROUPED_PRODUCT:
                    $collection = $product->getTypeInstance(true)->getAssociatedProducts($product);

                    foreach ($collection as $item) {
                        $childProductId = $item->getId();
                        $childProductDetails = $this->_productFactory->create()->load($item->getId());
                        if ($childProductDetails->getVisibility() == 1) {
                            $childProductData[$childProductId] = $this->_trackingHelper->getProductData($childProductDetails);
                            $childProductData[$childProductId]['parent_id'] = $product->getId();
                        }
                    }
                    $products[$id]['child_items'] = $childProductData;
                    $products[$id]['parent_id'] = $product->getId();
                    break;
            }

            if ($custOptions = $this->_productFactory->create()->load($product->getId())->getOptions()) {
                $customOptions = $this->_trackingHelper->productOptions($custOptions);
            }
            $options = array_merge($configOptions, $customOptions);

            if (!empty($options)) {
                $products[$id]['attributes'] = $options;
            }
        }

        return $products;
    }
}
