<?php

/**
 * @author Targetbay Team
 * @copyright Copyright (c) 2016 Targetbay
 * @package Targetbay_Tracking
 */

namespace Targetbay\Tracking\Model;

use Targetbay\Tracking\Api\OrderRepoInterface;

/**
 * Defines the implementation class of the OrderRepoInterface.
 */
class OrderRepo implements OrderRepoInterface
{
    const BILLING = 'billing';
    const SHIPPING = 'shipping';
    const ORDER_COMPLETE = 'complete';
    const ANONYMOUS_USER = 'anonymous';

    /**
     * @var \Targetbay\Tracking\Helper\Data
     */
    public $trackingHelper;

    /**
     * @var \Magento\Directory\Api\CountryInformationAcquirerInterface
     */
    public $countryInformation;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory
     */
    protected $orderCollection;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface $storeInfo
     */
    protected $storeInfo;

    protected $request;

    /**
     * @param \Targetbay\Tracking\Helper\Data $trackingHelper
     * @param \Magento\Directory\Api\CountryInformationAcquirerInterface $countryInformation
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollection
     * @param \Magento\Store\Model\StoreManagerInterface $storeInfo
     */
    public function __construct(
        \Magento\Directory\Api\CountryInformationAcquirerInterface $countryInformation,
        \Targetbay\Tracking\Helper\Data $trackingHelper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollection,
        \Magento\Store\Model\StoreManagerInterface $storeInfo,
        \Magento\Framework\App\RequestInterface $request
    ) {
        $this->_trackingHelper = $trackingHelper;
        $this->_countryInformation = $countryInformation;
        $this->_scopeConfig = $scopeConfig;
        $this->_orderCollection = $orderCollection;
        $this->_storeInfo = $storeInfo;
        $this->request = $request;
    }

    /**
     * Get the list of orders
     *
     * @return OrderInterface[]
     */
    public function getList(\Magento\Framework\Api\SearchCriteria $searchCriteria)
    {
        $storeId = $this->request->getParam('store_id');
        $collection = $this->_orderCollection->create()->addFieldToSelect('*');

        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());
        $collection->addFieldToFilter('store_id', $storeId);

        $ordersData = [];

        $timezone = $this->_scopeConfig->getValue('general/locale/timezone', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        foreach ($collection->getItems() as $order) {
            $ordersData [$order->getId()] = $order->toArray();
            $storeId = $order->getStoreId();
            $ordersData [$order->getId()]['store_id'] = $storeId;
            $ordersData [$order->getId()]['website_id'] = $this->_storeInfo->getStore($storeId)->getWebsiteId();
            $ordersData[$order->getId()]['timezone'] = $timezone;
            $ordersData [$order->getId()] [self::BILLING] = $this->getUserAddressData($order, self::BILLING);
            if ($order->getShippingAddress()) {
                $ordersData [$order->getId()] [self::SHIPPING] = $this->getUserAddressData($order, self::SHIPPING);
                if ($order->getStatus() == self::ORDER_COMPLETE) {
                    //order shipped date
                    foreach ($order->getShipmentsCollection() as $shipment) {
                        /** @var $shipment Mage_Sales_Model_Order_Shipment */
                        $ordersData[$order->getId()]['shipped_at'] = $shipment->getCreatedAt();
                        $ordersData[$order->getId()]['timezone'] = $timezone;
                    }
                }
            } else {
                $ordersData[$order->getId()][self::SHIPPING] = '';
            }
            if ($order->getPayment() !== null) {
                $ordersData [$order->getId()]['payment_method'] = $order->getPayment()->getMethodInstance()->getTitle();
            }
            
            $ordersData [$order->getId()] ['cart_items'] = $this->_trackingHelper->getOrderItemsInfo($order, true);
        }

        return $ordersData;
    }

    public function getUserAddressData($object, $type)
    {
        try {
            $address = ($type === self::SHIPPING) ? $object->getShippingAddress() : $object->getBillingAddress();
            if ($address !== null) {
                if ($address->getFirstname() !== '' && $address->getLastname() !== '') {
                    $addressData['first_name'] = $address->getFirstname();
                    $addressData['last_name'] = $address->getLastname();
                    $guestUsername = $address->getFirstname() . ' ' . $address->getLastname();
                    $gName = !empty($guestUsername) ? $guestUsername : self::ANONYMOUS_USER;
                    $addressData['user_name'] = $object->getCustomerIsGuest() ? $gName : $addressData['first_name'] . ' ' . $addressData['last_name'];
                    $addressData['order_id'] = $object->getId();
                    $addressData['user_mail'] = $object->getCustomerEmail();
                    $addressData['address1'] = $address->getStreet(1);
                    $addressData['address2'] = $address->getStreet(2);
                    $addressData['city'] = $address->getCity();
                    $addressData['state'] = $address->getRegion();
                    $addressData['zipcode'] = $address->getPostcode();

                    $countryName = '';
                    if ($address->getCountryId()) {
                        $countryName = $this->_countryInformation->getCountryInfo($address->getCountryId())->getFullNameLocale();
                    }

                    $addressData['country'] = $countryName !== '' ? $countryName : $address->getCountryId();
                    $addressData['phone'] = $address->getTelephone();

                    return $addressData;
                }
            }
        } catch (\Exception $e) {
            $this->_trackingHelper->debug(__($e->getMessage()));
        }
    }
}
